/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "../../libs.h"
#include "../engine/thread.h"
#include "texture.h"
#include "font.h"
#include "shader.h"
#include "scaledtexture.h"

#define ASSET_MANAGER_ITEMS_MAX 64
#define ASSET_MANAGER_ITEM_NAME_MAX 96
#define ASSET_MANAGER_HOLDERS_MAX 8

/** States */
#define ASSET_MANAGER_STATE_NOT_READY 0x00
#define ASSET_MANAGER_STATE_PENDING 0x01
#define ASSET_MANAGER_STATE_ASYNC_LOADING 0x02
#define ASSET_MANAGER_STATE_ASYNC_ERROR 0x03
#define ASSET_MANAGER_STATE_ASYNC_DONE 0x04
#define ASSET_MANAGER_STATE_SYNC_LOADING 0x05
#define ASSET_MANAGER_STATE_SYNC_ERROR 0x06
#define ASSET_MANAGER_STATE_SYNC_DONE 0x07

/** Type IDs */
#define ASSET_MANAGER_TYPE_TEXTURE 0x00
#define ASSET_MANAGER_TYPE_FONT 0x01
#define ASSET_MANAGER_TYPE_SHADER 0x02
#define ASSET_MANAGER_TYPE_SCALED_TEXTURE 0x03

// Owner
typedef uint8_t assetmanagerowner_t;

// Union of all the manager types
typedef union {
  assetmanagertexture_t texture;
  assetmanagershader_t shader;
  assetmanagerfont_t font;
  assetmanagerscaledtexture_t scaledTexture;
} assetmanagerassetdata_t;

// Item Type
typedef struct {
  uint8_t type;
  uint8_t state;
  char key[ASSET_MANAGER_ITEM_NAME_MAX];
  assetmanagerassetdata_t data;
  assetmanagerowner_t holders[ASSET_MANAGER_HOLDERS_MAX];
  uint8_t holderCount;
} assetmanageritem_t;

// Loader
typedef bool assetmanagerloader_t(assetmanageritem_t *item);

// Loader Definition
typedef struct {
  assetmanagerloader_t *loadAsync;
  assetmanagerloader_t *loadSync;
  assetmanagerloader_t *dispose;
} assetmanagerloaderdefinition_t;



// Manager
typedef struct {
  thread_t thread;
  bool finished;
  bool running;

  assetmanageritem_t items[ASSET_MANAGER_ITEMS_MAX];
  uint8_t itemCount;

  assetmanagerowner_t holders[ASSET_MANAGER_HOLDERS_MAX];
  uint8_t holderCount;
} assetmanager_t;